import copy, logging

class StoryDatabase(object):

    # callbacks you can register listeners against
    EVENT_PRE_STORY_SAVE = "preStorySave"
    EVENT_POST_STORY_SAVE = "postStorySave"

    def __init__(self):
        self._logger = logging.getLogger(__name__)

    def connect(self, db_name, host, port, username, password):
        raise NotImplementedError("Subclasses should implement this!")

    def storyExists(self, story_id):
        raise NotImplementedError("Subclasses should implement this!")

    def addStoryFromSentences(self, story_sentences, extra_attributes={}):
        '''
        Save a story based on it's sentences to the database.  Return success or failure boolean.
        This is pairs well with mediacloud.sentencesMatchingByStory(...).  This saves or updates.
        '''
        # if nothing to save, bail
        if len(story_sentences)==0:
            return False
        # verify all the sentences are part of the same story
        stories_id_list = set( [ s['stories_id'] for s in story_sentences ] )
        if len(stories_id_list)>1:
            raise Exception('Expecting all the sentences to be part of the same story (ie. one entry from mediacloud.sentencesMatchingByStory)')
        stories_id = list(stories_id_list)[0]
        # save or update the story
        sentences_by_number = {str(s['sentence_number']):s['sentence'] for s in sorted(story_sentences, key=lambda x: x['sentence_number'], reverse=True)}
        if not self.storyExists(stories_id):
            # if the story is new, save it all
            story_attributes = {
                'stories_id': stories_id,
                'media_id': story_sentences[0]['media_id'],
                'publish_date': story_sentences[0]['publish_date'],
                'language': story_sentences[0]['language'],
                'story_sentences': sentences_by_number,
                'story_sentences_count': len(sentences_by_number)
            }
            self._saveStory( dict(story_attributes.items() + extra_attributes.items()) )
        else:
            # if the story exists already, add any new sentences
            story = self.getStory(stories_id)
            all_sentences = dict(story['story_sentences'].items() + sentences_by_number.items())
            story_attributes = {
                'stories_id': stories_id,
                'story_sentences': all_sentences,
                'story_sentences_count': len(all_sentences)
            }
            self._updateStory( dict(story_attributes.items() + extra_attributes.items()) )
        return True

    def updateStory(self, story, extra_attributes={}):
        # if it is a new story, just add it normally
        if not self.storyExists(story['stories_id']):
            return self.addStory(story,extra_attributes)
        else:
            story_to_save = copy.deepcopy( story )
            story_to_save = dict(story_to_save.items() + extra_attributes.items())
            story_to_save['stories_id'] = story['stories_id']
            if 'story_sentences' in story:
                story_to_save['story_sentences_count'] = len(story['story_sentences'])
            self._updateStory(story_to_save)
            saved_story = self.getStory( story['stories_id'] )
            self._logger.debug('Updated '+str(story['stories_id']))

    def addStory(self, story, extra_attributes={}):
        ''' 
        Save a story (python object) to the database. This does NOT update stories.
        Return success or failure boolean.
        '''
        if self.storyExists(story['stories_id']):
            self._logger.warn('Not saving '+str(story['stories_id'])+' - already exists')
            return False
        story_to_save = copy.deepcopy( story )
        story_to_save = dict(story_to_save.items() + extra_attributes.items())
        story_to_save['_stories_id'] = story['stories_id']
        if 'story_sentences' in story:
            story_to_save['story_sentences_count'] = len(story['story_sentences'])
        self._saveStory( story_to_save )
        saved_story = self.getStory( story['stories_id'] )
        self._logger.debug('Saved '+str(story['stories_id']))
        return True

    def _updateStory(self, story_attributes):
        raise NotImplementedError("Subclasses should implement this!")

    def _saveStory(self, story_attributes):
        raise NotImplementedError("Subclasses should implement this!")

    def getStory(self, story_id):
        raise NotImplementedError("Subclasses should implement this!")

    def storyCount(self):
        raise NotImplementedError("Subclasses should implement this!")        

    def createDatabase(self, db_name):
        raise NotImplementedError("Subclasses should implement this!")
        
    def deleteDatabase(self, db_name):
        raise NotImplementedError("Subclasses should implement this!")
        
    def getMaxStoryId(self):
        raise NotImplementedError("Subclasses should implement this!")

    def initialize(self):
        raise NotImplementedError("Subclasses should implement this!")

class MongoStoryDatabase(StoryDatabase):

    def __init__(self, db_name=None, host='127.0.0.1', port=27017, username=None, password=None):
        super(MongoStoryDatabase, self).__init__()
        import pymongo
        self._server = pymongo.MongoClient(host, port)
        if db_name is not None:
            self.selectDatabase(db_name)

    def createDatabase(self, db_name):
        self.selectDatabase(db_name)

    def selectDatabase(self, db_name):
        self._db = self._server[db_name]

    def deleteDatabase(self, ignored):
        self._db.drop_collection('stories')

    def storyCount(self):
        self._db.stories.count();

    def storyExists(self, story_id):
        story = self.getStory(story_id)
        return story != None

    def _updateStory(self, story_attributes):
        story = self.getStory(story_attributes['stories_id'])
        story_attributes['_id'] = story['_id']
        story_id = self._db.stories.save(story_attributes)
        story = self.getStory(story_attributes['stories_id'])
        return story

    def _saveStory(self, story_attributes):
        story_db_id = self._db.stories.insert(story_attributes)
        story = self.getStory(story_attributes['stories_id'])
        return story

    def getStory(self, story_id):
        stories = self._db.stories.find( { "stories_id": story_id } ).limit(1)
        if stories.count()==0:
            return None
        return stories.next()

    def getMaxStoryId(self):
        max_story_id = 0
        if self._db.stories.count() > 0 :
            max_story_id = self._db.stories.find().sort("stories_id",-1)[0]['stories_id']
        return int(max_story_id)

    def initialize(self):
        # nothing to init for mongo
        return

    def storyCount(self):
        return self._db['stories'].count()



import os.path
import codecs

TEST_USER_EMAIL = "mc-api-test@media.mit.edu"

QUERY_LAST_FEW_DAYS = "publish_date:[NOW-3DAY TO NOW]"
QUERY_LAST_WEEK = "publish_date:[NOW-7DAY TO NOW]"
QUERY_LAST_MONTH = "publish_date:[NOW-31DAY TO NOW]"
QUERY_LAST_YEAR = "publish_date:[NOW-1YEAR TO NOW]"
QUERY_LAST_DECADE = "publish_date:[NOW-10YEAR TO NOW]"
QUERY_ENGLISH_LANGUAGE = "language:en"

basedir = os.path.abspath(os.path.dirname(os.path.dirname(os.path.dirname(__file__))))


def load_text_from_fixture(filename):
    f = codecs.open(os.path.join(basedir, "mediacloud", "test", "fixtures", filename), 'r',
                    encoding='utf8')
    text = f.read()
    return text




# Tag set ids for metadata about media in our system
TAG_SET_PUBLICATION_COUNTRY = 1935  # holds the country of publication of a source
TAG_SET_PUBLICATION_STATE = 1962  # holds the state of publication of a source (only US and India right now)
TAG_SET_PRIMARY_LANGUAGE = 1969  # holds the primary language of a source
TAG_SET_COUNTRY_OF_FOCUS = 1970  # holds the primary focus on what country for a source
TAG_SET_MEDIA_TYPE = 1972  # holds what type of media source this is (broadcast, online, etc)

# Tag set ids for information about a story
TAG_SET_NYT_THEMES = 1963  # the tag set the top 600 labels from our NYT-corpus-trained model
TAG_SET_CLIFF_PLACES = 1011  # the tag set all the geographic country and state tags are in
TAG_SET_CLIFF_ORGS = 2388  # the tag set with organizational entities
TAG_SET_CLIFF_PEOPLE = 2389  # the tag set with people entities

# Tag set ids for metadata about stories in our system
TAG_SET_DATE_GUESS_METHOD = 508
TAG_SET_EXTRACTOR_VERSION = 1354
TAG_SET_GEOCODER_VERSION = 1937
TAG_CLIFF_CLAVIN_2_3_0 = 9353691  # the tag that indicates a story was tagged by the CLIFF version 2.3.0
TAG_CLIFF_CLAVIN_2_4_1 = 9696677  # the tag that indicates a story was tagged by the CLIFF version 2.4.1
TAG_SET_NYT_THEMES_VERSION = 1964
TAG_NYT_LABELER_1_0_0 = 9360669  # the tag that indicates a story was tagged by the NYT labeller version 1

# Tag ids for information about a story
TAG_STORY_UNDATEABLE = 8877812  # if a story has this tag, that means it was undateable


'''
These are helpers for calling media/story tag update methods.  These wrappers exist
because you can specify this by tag/tag-set name, or by tag-id.  Because you don't want to mess
these calls up, I found it helpful to fomarlize this, instead of just accepting a dict from the
caller (which might be missing params).
'''

TAG_ACTION_ADD = 'add'
TAG_ACTION_REMOVE = 'remove'


class TagSpec(object):

    def __init__(self, tag_set_name=None, tag_name=None, action=TAG_ACTION_ADD, tags_id=None):
        self.tag_set_name = tag_set_name
        self.tag_name = tag_name
        self.tags_id = tags_id
        self.action = action

    def isSpecifiedByName(self):
        return (self.tag_name is not None) and (self.tag_set_name is not None)

    def isSpeficiedById(self):
        return self.tags_id is not None

    def getBaseParams(self):
        params = {
            'action': self.action
        }
        if self.isSpeficiedById():
            params['tags_id'] = self.tags_id
        elif self.isSpecifiedByName():
            params['tag'] = self.tag_name
            params['tag_set'] = self.tag_set_name
        return params


class StoryTag(TagSpec):

    def __init__(self, stories_id, tag_set_name=None, tag_name=None, action=TAG_ACTION_ADD, tags_id=None):
        self.stories_id = stories_id
        super(StoryTag, self).__init__(tag_set_name, tag_name, action, tags_id)

    def getParams(self):
        params = {'stories_id': self.stories_id }
        params.update(self.getBaseParams())
        return params


class MediaTag(TagSpec):

    def __init__(self, media_id, tag_set_name=None, tag_name=None, action=TAG_ACTION_ADD, tags_id=None):
        self.media_id = media_id
        super(MediaTag, self).__init__(tag_set_name, tag_name, action, tags_id)

    def getParams(self):
        params = {'media_id': self.media_id }
        params.update(self.getBaseParams())
        return params
